/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the MLdonkey protocol handler 2.2.
 *
 * The Initial Developer of the Original Code is
 * Simon Peter <dn.tlp@gmx.net>.
 * Portions created by the Initial Developer are Copyright (C) 2003 - 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Sven Koch
 * Len Walter <len@unsw.edu.au>
 * Dan Fritz <templar_of_ni@yahoo.se>
 * David Ciecierski <dawid.ciecierski@gmail.com>
 * Dennis Plger <dennis@dieploegers.de>
 * Dominik Rttsches <d-r@roettsches.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/***** Defines *****/

// components defined in this file
const ED2KPROT_HANDLER_CONTRACTID =
    "@mozilla.org/network/protocol;1?name=ed2k";
const ED2KPROT_HANDLER_CID =
    Components.ID("{af8d664a-d002-438f-84a3-01f3a8ff325b}");

const MAGNETPROT_HANDLER_CONTRACTID =
    "@mozilla.org/network/protocol;1?name=magnet";
const MAGNETPROT_HANDLER_CID =
    Components.ID("{3e022170-58b0-4548-ba4c-1f47d54c7767}");

const SIG2DATPROT_HANDLER_CONTRACTID =
    "@mozilla.org/network/protocol;1?name=sig2dat";
const SIG2DATPROT_HANDLER_CID =
    Components.ID("{2a2e71ea-e857-4c71-9c93-04ff681df88a}");

const TORRENTPROT_HANDLER_CONTRACTID =
    "@simonpeter.de/mldonkeyhandler;1?name=torrent";
const TORRENTPROT_HANDLER_CID =
    Components.ID("{51401ffb-1b9c-46f1-b1e8-b8a6cf77199b}");

// components used in this file
const NS_IOSERVICE_CID = "{9ac9e770-18bc-11d3-9337-00104ba0fd40}";
const NS_PREFSERVICE_CONTRACTID = "@mozilla.org/preferences-service;1";
const URI_CONTRACTID = "@mozilla.org/network/simple-uri;1";
const NS_WINDOWWATCHER_CONTRACTID = "@mozilla.org/embedcomp/window-watcher;1";
const INPUTSTREAMCHANNEL_CONTRACTID = "@mozilla.org/network/input-stream-channel;1";
const CATEGORY_MANAGER_CONTRACTID = "@mozilla.org/categorymanager;1";
const HTTP_HANDLER_CONTRACTID = "@mozilla.org/network/protocol;1?name=http";

// interfaces used in this file
const nsIProtocolHandler    = Components.interfaces.nsIProtocolHandler;
const nsIURI                = Components.interfaces.nsIURI;
const nsIPrefService        = Components.interfaces.nsIPrefService;
const nsIWindowWatcher      = Components.interfaces.nsIWindowWatcher;
const nsIChannel            = Components.interfaces.nsIChannel;
const nsIContentPolicy      = Components.interfaces.nsIContentPolicy;

// some misc. constants
const PREF_BRANCH   = "extensions.mldonkey.";
const WND_WIDTH = 320;
const WND_HEIGHT = 200;

const WND_EMULE_WIDTH = 450;
const WND_EMULE_HEIGHT = 75;

const WND_AMULE_WIDTH = 320;
const WND_AMULE_HEIGHT = 200;

// configuration (and defaults)
cfgServer           = "localhost";
cfgPort             = "4080";
myWnd               = null;
cfgMode             = "mldonkey";
cfgPass             = "";
cfgProtocols        = "ed2k magnet sig2dat torrent";
cfgAlwaysOpenWindow = true;

/***** MLdonkeyProtocolHandler *****/

function MLdonkeyProtocolHandler(scheme)
{
    this.scheme = scheme;
}

// attribute defaults
MLdonkeyProtocolHandler.prototype.defaultPort = -1;
MLdonkeyProtocolHandler.prototype.protocolFlags = nsIProtocolHandler.URI_NORELATIVE;
MLdonkeyProtocolHandler.prototype.withinLoad = false;

MLdonkeyProtocolHandler.prototype.allowPort = function(aPort, aScheme)
{
    return false;
}

MLdonkeyProtocolHandler.prototype.newURI = function(aSpec, aCharset, aBaseURI)
{
    var uri = Components.classes[URI_CONTRACTID].createInstance(nsIURI);
    uri.spec = aSpec;
    return uri;
}

MLdonkeyProtocolHandler.prototype.loadMLDonkey = function(aURI)
{
    var myURI = "";
    var myTitle = "";
    var myWidth = WND_WIDTH;
    var myHeight = WND_HEIGHT;
    var myURIComponent = encodeURIComponent(decodeURI(aURI.spec));

    if(cfgMode == "mldonkey") {
        // MLDonkey mode
        myTitle = "MLDonkey";

        // rewrite the URI into a http URL to the mldonkey client
        myURI = "http://";
        //    if(cfgUser != "") myURI += cfgUser + ":" + cfgPass + "@";
        myURI += cfgServer + ":" + cfgPort + "/submit?q=dllink+" + myURIComponent;
    } else if(cfgMode == "emule") {
        // eMule mode
        myTitle = "eMule";
        myWidth = WND_EMULE_WIDTH;
        myHeight = WND_EMULE_HEIGHT

        // rewrite the URI into a http URL to the eMule client
        myURI = "http://" + cfgServer + ":" + cfgPort + "/?w=password&p=" + cfgPass;
        myURI += "&cat=0&c=" + myURIComponent;
    } else if(cfgMode == "amule") {
        // aMule mode
        myTitle = "aMule";
        myWidth = WND_AMULE_WIDTH;
        myHeight = WND_AMULE_HEIGHT;

        // rewrite the URI into a http URL to the aMule client
        myURI = "http://" + cfgServer + ":" + cfgPort + "/footer.php?pass=" + cfgPass;
        myURI += "&Submit=Download+link&ed2klink=" + myURIComponent + "&selectcat=all";
    }

    if(cfgAlwaysOpenWindow == true || this.withinLoad) {
        // open up a window with our newly generated http URL
        var wwatch = Components.classes[NS_WINDOWWATCHER_CONTRACTID].getService(nsIWindowWatcher);
        if(myWnd == null || myWnd.closed == true) {
            myWnd = wwatch.openWindow(wwatch.activeWindow, myURI, myTitle,
                                      "width=" + myWidth + ", height=" + myHeight, null);
        } else
            myWnd.location.href = myURI;

        return null;
    } else {
        // create a new HTTP channel with our generated URL
        var httphandler = Components.classes[HTTP_HANDLER_CONTRACTID].getService(nsIProtocolHandler);
        var httpURI = httphandler.newURI(myURI, "UTF-8", null);
        return httphandler.newChannel(httpURI);
    }
}

MLdonkeyProtocolHandler.prototype.shouldLoad = function(ct, cl, org, ctx, mt, ext)
{
    // If URI ends in .torrent, pass it to MLDonkey instead of loading
    if(ct == nsIContentPolicy.TYPE_DOCUMENT && cl.spec.match(/\.torrent$/) &&
       !this.withinLoad) {
        this.withinLoad = true;
        this.loadMLDonkey(cl);
        this.withinLoad = false;
        return nsIContentPolicy.REJECT_REQUEST;
    }

    return nsIContentPolicy.ACCEPT;
}

MLdonkeyProtocolHandler.prototype.shouldProcess = function(ct, cl, org, ctx, mt, ext)
{
//     dump("shouldProcess(" + ct + ", " + cl + ", " + org + ", " + ctx + ", " + mt + ", " + ext + ")\n");
    return nsIContentPolicy.ACCEPT;
}

MLdonkeyProtocolHandler.prototype.newChannel = function(aURI)
{
    // pass URI to MLDonkey
    var chan = this.loadMLDonkey(aURI);

    // return a fake empty channel so current window doesn't change
    if(chan == null) chan = Components.classes[INPUTSTREAMCHANNEL_CONTRACTID].createInstance(nsIChannel);
    return chan;
}

/***** MLdonkeyProtocolHandlerFactory *****/

function MLdonkeyProtocolHandlerFactory(scheme)
{
    this.scheme = scheme;
}

MLdonkeyProtocolHandlerFactory.prototype.createInstance = function(outer, iid)
{
    if(outer != null) throw Components.results.NS_ERROR_NO_AGGREGATION;

    if(!iid.equals(nsIProtocolHandler) && !iid.equals(nsIContentPolicy))
        throw Components.results.NS_ERROR_INVALID_ARG;

    return new MLdonkeyProtocolHandler(this.scheme);
}

var factory_ed2k = new MLdonkeyProtocolHandlerFactory("ed2k");
var factory_magnet = new MLdonkeyProtocolHandlerFactory("magnet");
var factory_sig2dat = new MLdonkeyProtocolHandlerFactory("sig2dat");
var factory_torrent = new MLdonkeyProtocolHandlerFactory("torrent");

/***** Ed2kzillaModule *****/

var Ed2kzillaModule = new Object();

Ed2kzillaModule.readPreferences = function(pref_branch)
{
    // get preferences branch
    var PrefService = Components.classes[NS_PREFSERVICE_CONTRACTID].getService(nsIPrefService);
    var myPrefs = PrefService.getBranch(null);  // Mozilla bug #107617

    // read preferences (if available)
    if(myPrefs.getPrefType(pref_branch + "server") == myPrefs.PREF_STRING)
      cfgServer = myPrefs.getCharPref(pref_branch + "server");
    if(myPrefs.getPrefType(pref_branch + "port") == myPrefs.PREF_STRING)
      cfgPort = myPrefs.getCharPref(pref_branch + "port");
    if(myPrefs.getPrefType(pref_branch + "mode") == myPrefs.PREF_STRING)
      cfgMode = myPrefs.getCharPref(pref_branch + "mode");
    if(myPrefs.getPrefType(pref_branch + "pass") == myPrefs.PREF_STRING)
      cfgPass = myPrefs.getCharPref(pref_branch + "pass");
    if(myPrefs.getPrefType(pref_branch + "protocols") == myPrefs.PREF_STRING)
      cfgProtocols = myPrefs.getCharPref(pref_branch + "protocols");
    if(myPrefs.getPrefType(pref_branch + "alwaysopenwindow") == myPrefs.PREF_BOOL)
      cfgAlwaysOpenWindow = myPrefs.getBoolPref(pref_branch + "alwaysopenwindow");
}

Ed2kzillaModule.registerSelf = function(compMgr, fileSpec, location, type)
{
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    // register ed2k protocol handler
    compMgr.registerFactoryLocation(ED2KPROT_HANDLER_CID,
                                    "ED2K protocol handler",
                                    ED2KPROT_HANDLER_CONTRACTID,
                                    fileSpec, location, type);

    // register magnet protocol handler
    compMgr.registerFactoryLocation(MAGNETPROT_HANDLER_CID,
                                    "Magnet protocol handler",
                                    MAGNETPROT_HANDLER_CONTRACTID,
                                    fileSpec, location, type);

    // register sig2dat protocol handler
    compMgr.registerFactoryLocation(SIG2DATPROT_HANDLER_CID,
                                    "Sig2dat protocol handler",
                                    SIG2DATPROT_HANDLER_CONTRACTID,
                                    fileSpec, location, type);

    // register torrent content policy
    compMgr.registerFactoryLocation(TORRENTPROT_HANDLER_CID,
                                    "torrent content policy",
                                    TORRENTPROT_HANDLER_CONTRACTID,
                                    fileSpec, location, type);
    var catMgr = Components.classes[CATEGORY_MANAGER_CONTRACTID].
    getService(Components.interfaces.nsICategoryManager);
    catMgr.addCategoryEntry("content-policy", TORRENTPROT_HANDLER_CONTRACTID,
                            TORRENTPROT_HANDLER_CONTRACTID, true, true);
}

Ed2kzillaModule.unregisterSelf = function(compMgr, fileSpec, location)
{
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    // unregister our components
    compMgr.unregisterFactoryLocation(ED2KPROT_HANDLER_CID, fileSpec);
    compMgr.unregisterFactoryLocation(MAGNETPROT_HANDLER_CID, fileSpec);
    compMgr.unregisterFactoryLocation(SIG2DATPROT_HANDLER_CID, fileSpec);
    compMgr.unregisterFactoryLocation(TORRENTPROT_HANDLER_CID, fileSpec);

    // unregister torrent content policy
    var catMgr = Components.classes[CATEGORY_MANAGER_CONTRACTID].
    getService(Components.interfaces.nsICategoryManager);
    catMgr.deleteCategoryEntry("content-policy", TORRENTPROT_HANDLER_CONTRACTID, true);
}

Ed2kzillaModule.getClassObject = function(compMgr, cid, iid)
{
//     dump("getClassObject(" + cid + ", " + iid + ")\n");

    if(!iid.equals(Components.interfaces.nsIFactory))
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    // read preferences
    this.readPreferences(PREF_BRANCH);

    // return protocol handler factories
    if(cid.equals(ED2KPROT_HANDLER_CID)) { if(cfgProtocols.search("ed2k") != -1) return factory_ed2k; else return null; }
    if(cid.equals(MAGNETPROT_HANDLER_CID)) { if(cfgProtocols.search("magnet") != -1) return factory_magnet; else return null; }
    if(cid.equals(SIG2DATPROT_HANDLER_CID)) { if (cfgProtocols.search("sig2dat") != -1) return factory_sig2dat; else return null; }
    if(cid.equals(TORRENTPROT_HANDLER_CID)) { if (cfgProtocols.search("torrent") != -1) return factory_torrent; else return null; }

    throw Components.results.NS_ERROR_NO_INTERFACE;
}

Ed2kzillaModule.canUnload = function(compMgr)
{
    return true;    // our objects can be unloaded
}

/***** Entrypoint *****/

function NSGetModule(compMgr, fileSpec)
{
    return Ed2kzillaModule;
}
